//
//  CollisionManager.cpp
//  GlobalGameJam17
//
//  Created by Cesar Parent on 21/01/2017.
//  Copyright © 2017 teamrocket. All rights reserved.
//

#include "CollisionManager.hpp"
#include "GameObject.hpp"


CollisionManager& CollisionManager::instance() {
    static CollisionManager instance_;
    return instance_;
}

CollisionManager::CollisionManager() {
    
}

void CollisionManager::addObject(ARC<GameObject> object) {
    objects_.push_back(object);
}

void CollisionManager::removeObject(ARC<GameObject> object) {
    auto it = std::remove(objects_.begin(), objects_.end(), object);
    objects_.erase(it, objects_.end());
}

void CollisionManager::removeAll() {
    objects_.clear();
}

void CollisionManager::checkCollisions() {
    
    for(size_t i = 0; i < objects_.size(); ++i) {
        auto a = objects_[i];
        if(a->categoryMask == 0) continue;
        
        for(size_t j = i+1; j < objects_.size(); ++j) {
            auto b = objects_[j];
            if(b->categoryMask == 0) continue;
            if((a->collisionMask & b->categoryMask)) {
                if(a->collider().intersectionWithRect(b->collider()) != Rect::nullRect) {
                    a->didCollideWith(b);
                }
            }
            else if((b->collisionMask & a->categoryMask)) {
                if(a->collider().intersectionWithRect(b->collider()) != Rect::nullRect) {
                    b->didCollideWith(a);
                }
            }
        }
    }
    
}

