//
//  Enemy.cpp
//  GlobalGameJam17
//
//  Created by Cesar Parent on 21/01/2017.
//  Copyright © 2017 teamrocket. All rights reserved.
//

#include "Enemy.hpp"
#include "Utils.hpp"

Enemy::Enemy(const std::string& sprite, float baseLine)
: GameObject(sprite)
, speed_(Constants::ENEMY_SPEED)
, phase_(0)
, baseLine_(baseLine)
, direction_(Direction::LEFT) {
    frequency_ = Utils::random<float>(1.f, 10.f);
    amplitude_ = Utils::random<float>(1.f, 6.f);
    
    trail_ = PlistDecoder::decodeObjectFromFile<EmitterNode>("assets/objects/trail.plist");
}

Enemy::~Enemy() {
    
}

void Enemy::destroy() {
    NotificationCenter::defaultCenter().postNotificationWithName(Notif::EXPLOSION, nullptr);
    auto expl = PlistDecoder::decodeObjectFromFile<EmitterNode>("assets/objects/explode.plist");
    expl->setPosition(position());
    expl->runAction(Action::sequence({
        Action::waitForDuration(.1f),
        Action::runBlock([=]{ expl->setEmit(false); }),
        Action::waitForDuration(1.f),
        Action::removeFromParent()
    }), "boom");
    parent()->addChild(expl);
    GameObject::destroy();
}

void Enemy::didCollideWith(ARC<GameObject> other) {
    GameObject::didCollideWith(other);
    other->destroy();
}

void Enemy::didEnterParent() {
    parent()->addChild(trail_);
}

void Enemy::willMoveFromParent() {
    GameObject::willMoveFromParent();
    trail_->setEmit(false);
    trail_->runAction(Action::sequence({
        Action::fadeAlphaTo(0.f, .5f),
        Action::removeFromParent()
    }), "fade");
}

void Enemy::update(double delta) {
    delta = 1.f/60.f;
    
    GameObject::update(delta);
    trail_->setPosition(position());
    //trail_->setEmissionSpeed(Constants::LEVEL_SPEED * InfRunner::instance().speedFactor());
    
    phase_ += (delta * frequency_ * InfRunner::instance().speedFactor());
    if(phase_ >= 2*M_PI) phase_ -= (2*M_PI);
    setY(baseLine_ + (std::sin(phase_) * amplitude_));
    setX(position().x + (direction_ * speed_ * delta  * InfRunner::instance().speedFactor()));
    
    if(position().x < 0) {
        removeFromParent();
    }
}
