//
//  Player.cpp
//  GlobalGameJam17
//
//  Created by Cesar Parent on 21/01/2017.
//  Copyright © 2017 teamrocket. All rights reserved.
//

#include "Player.hpp"
#include "Utils.hpp"
#include "Bullet.hpp"
#include "InfRunner.hpp"

Player::Player(float baseLine)
: GameObject("assets/sprites/main.png")
, grounded_(false)
, jumpControl_(Constants::JUMP_CONTROL)
, acceleration_(0)
, vspeed_(0)
, baseLine_(baseLine)
, nextShoot_(0) {
    controls.jump = false;
    controls.shoot = false;
    setAnchorPoint(.5f, 1.f);
    setName(Constants::PLAYER_TAG);
    
    setClipRect({0.f, 0.f, 12.f, 12.f});
    runAction(Action::repeatForever(Action::animate(__range<int>(0, 14), .05)), "run");
    
}

void Player::update(double delta) {
    if(InfRunner::instance().gameOver()) return;
    delta = (1.f/60.f) * InfRunner::instance().speedFactor();
    
    GameObject::update(delta);
    acceleration_ += Constants::GRAVITY;
    vspeed_ += acceleration_ * delta;
    setY(position().y + (vspeed_ * delta) + (0.5f * acceleration_ * (delta * delta)));
    
    acceleration_ = 0;
    
    if(position().y >= baseLine_) {
        vspeed_ = 0.f;
        grounded_ = true;
        setY(baseLine_);
    } else {
        grounded_ = false;
    }
    doJump_(delta);
    
    if(!controls.shoot)
        nextShoot_ -= delta * InfRunner::instance().speedFactor();
    if(nextShoot_ < 0) nextShoot_ = 0;
}

void Player::didCollideWith(ARC<GameObject> other) {
    GameObject::didCollideWith(other);
    other->destroy();
    NotificationCenter::defaultCenter().postNotificationWithName(Notif::EXPLOSION, nullptr);
    NotificationCenter::defaultCenter().postNotificationWithName(Notif::DEATH, nullptr);
}

void Player::destroy() {
    auto expl = PlistDecoder::decodeObjectFromFile<EmitterNode>("assets/objects/explode.plist");
    expl->setPosition(position());
    expl->runAction(Action::sequence({
        Action::waitForDuration(.1f),
        Action::runBlock([=]{ expl->setEmit(false); }),
        Action::waitForDuration(1.f),
        Action::removeFromParent()
    }), "boom");
    parent()->addChild(expl);
    GameObject::destroy();
    
    setHidden(true);
}

bool Player::canShoot() {
    if(controls.shoot && nextShoot_ <= 0) {
        nextShoot_ += Constants::SHOOT_INTERVAL;
        return true;
    }
    return false;
}

void Player::doJump_(double delta) {
    if(!controls.jump) {
        jumpControl_ = grounded_ ? Constants::JUMP_CONTROL : 0;
        return;
    }
    if(jumpControl_ <= 0) return;
    
    if(grounded_) {
        acceleration_ -= Constants::JUMP_IMPULSE / InfRunner::instance().speedFactor();
        AudioEngine::playSound("assets/audio/jump.wav");
    } else {
        acceleration_ -= Constants::JUMP_JETPACK;
    }
    jumpControl_ -= delta;
    return;
}
