//
//  ScoreManager.cpp
//  GlobalGameJam17
//
//  Created by Cesar Parent on 22/01/2017.
//  Copyright © 2017 teamrocket. All rights reserved.
//

#include <fstream>
#include <Meteor/Meteor.h>
#include "ScoreManager.hpp"
#include "json.hpp"

using namespace Meteor;
using json = nlohmann::json;

ScoreManager& ScoreManager::instance() {
    static ScoreManager instance_;
    return instance_;
}

ScoreManager::ScoreManager() {
    std::string path = FileManager::applicationSupportDirectoryURL() + "scores.json";
    std::ifstream infile{path};
    
    if(!infile.is_open()) return;
    
    json data;
    
    infile >> data;
    
    for(auto& item : data) {
        if(!item.is_number_integer()) continue;
        addScore(item);
    }
}

ScoreManager::~ScoreManager() {
    std::string path = FileManager::applicationSupportDirectoryURL() + "scores.json";
    std::ofstream outfile{path};
    
    if(!outfile.is_open()) return;
    
    json data;
    for(auto score : scores_) {
        data.push_back(score);
    }
    outfile << std::setw(4) << data << std::endl;
}

void ScoreManager::addScore(int score) {
    scores_.push_back(score);
    std::sort(scores_.begin(), scores_.end(), [](int a, int b) { return a > b; });
}

void ScoreManager::enumerateScores(ScoreEnumerator block, int limit) {
    for(int i : __range<int>(0, min<int>(limit, (int)scores_.size()))) {
        block(i+1, scores_[i]);
    }
}
