//
//  ScoreMenu.cpp
//  GlobalGameJam17
//
//  Created by Cesar Parent on 22/01/2017.
//  Copyright (c) 2017 teamrocket. All rights reserved.
//

#include "ScoreMenu.hpp"
#include "TunerScene.hpp"
#include "ScoreManager.hpp"

static float stackLevel(int level) {
    return level * 16.f;
}

ScoreMenu::ScoreMenu(int score) {
    setZSort(true);
    
    auto back = alloc<SpriteNode>("assets/sprites/mainmenu.png");
    back->setZPosition(0);
    addChild(back);
    
    auto gameover = PlistDecoder::decodeObjectFromFile<LabelNode>("assets/objects/label.plist");
    gameover->setVerticalAlignmentMode(LabelNode::VerticalAlignmentMode::Top);
    gameover->setText("Game Over - " + std::to_string(score));
    gameover->setY(stackLevel(-4));
    addChild(gameover);
    
    auto title = PlistDecoder::decodeObjectFromFile<LabelNode>("assets/objects/label.plist");
    title->setText("High Scores:");
    title->setY(stackLevel(-2));
    addChild(title);
    
    ScoreManager::instance().enumerateScores([this](int rank, int score) {
        
        auto rankLabel = PlistDecoder::decodeObjectFromFile<LabelNode>("assets/objects/label.plist");
        rankLabel->setHorizontalAlignmentMode(LabelNode::HorizontalAlignmentMode::Left);
        rankLabel->setY(stackLevel(rank - 2));
        rankLabel->setText("#"+std::to_string(rank));
        rankLabel->setX(-size().width/6.f);
        addChild(rankLabel);
        
        auto scoreLabel = PlistDecoder::decodeObjectFromFile<LabelNode>("assets/objects/label.plist");
        scoreLabel->setHorizontalAlignmentMode(LabelNode::HorizontalAlignmentMode::Right);
        scoreLabel->setY(stackLevel(rank - 2));
        scoreLabel->setText(std::to_string(score));
        scoreLabel->setX(size().width/6.f);
        addChild(scoreLabel);
        
    }, 3);
}

void ScoreMenu::didMoveToView() {
    Controls::setDelegate(nullptr);
    runAction(Action::sequence({
        Action::waitForDuration(1.f),
        Action::runBlock([this] {
            Controls::setDelegate(this);
            auto cont = PlistDecoder::decodeObjectFromFile<LabelNode>("assets/objects/label.plist");
            cont->setText("press any key to play again");
            cont->setY(stackLevel(3));
            addChild(cont);
        }),
    }), "unlock");
}

// MARK: ControlDelegate

void ScoreMenu::keyDown(Meteor::Key aKey) {
    start();
}

void ScoreMenu::controllerButtonDown(int controller, Button aButton) {
    start();
}

void ScoreMenu::start() {
    Game::instance().presentScene(alloc<TunerScene>(),
                                  Transition::fadeWithDuration(.5f));
}
