//
//  Constants.cpp
//  GlobalGameJam17
//
//  Created by Cesar Parent on 21/01/2017.
//  Copyright © 2017 teamrocket. All rights reserved.
//

#include <fstream>
#include "Utils.hpp"
#include "json.hpp"

using json = nlohmann::json;

namespace Constants {
    const std::string PLAYER_TAG = "PLAYER_TAG";
    const std::string TICK_TAG = "TICK_TAG";
    const std::string ENEMY_TAG = "ENEMY_TAG";
    const Color OVERLAY_COLOUR{0.247, 0.192, 0.109};
    
    float ENEMY_SPEED = 110.f;
    float LEVEL_SPEED = 80.f;
    float MAX_SPEED_MULT = 3.f;
    
    float GRAVITY = 200.f;
    float JUMP_IMPULSE = 3500.f;
    float JUMP_JETPACK = 250.f;
    float JUMP_CONTROL = 0.14f;
    
    float SHOOT_INTERVAL = 0.5f;
    
    float NEEDLE_SPEED = 0.5f;
    int SCORE_PER_SECOND = 10;
    
    template <typename T>
    void _load(const json& data, const std::string& key, T& value) {
        if(data.find(key) == data.end()) return;
        value = data[key];
    }
    
    void reload() {
        std::ifstream infile{"assets/settings.json"};
        if(!infile.is_open()) return;
        
        json data;
        infile >> data;
        
        _load(data, "enemy_speed", ENEMY_SPEED);
        _load(data, "level_speed", LEVEL_SPEED);
        _load(data, "max_speed_mult", MAX_SPEED_MULT);
        _load(data, "gravity", GRAVITY);
        _load(data, "jump_impulse", JUMP_IMPULSE);
        _load(data, "jump_continue", JUMP_JETPACK);
        _load(data, "jump_control_time", JUMP_CONTROL);
        _load(data, "shoot_interval", SHOOT_INTERVAL);
        _load(data, "needle_speed", NEEDLE_SPEED);
        _load(data, "score_per_second", SCORE_PER_SECOND);
    }
}

namespace Notif {
    const char* DEATH = "Notif::DEATH";
    const char* EXPLOSION = "Notif::EXPLOSION";
}
