//
//  JSONMap.hpp
//  RainbowJam16
//
//  Created by Cesar Parent on 21/08/2016.
//  Copyright (c) 2016 cesarparent. All rights reserved.
//
#pragma once
#include <map>
#include <functional>

#include "../Nodes.h"
#include "MapOptions.h"
#include "MapLayer.h"
#include "MapObject.h"

NS_BEGIN

class Map : public Node {
public:
    
    typedef std::function<void(const MapObject&, bool&)> ObjectBlock;
    
    typedef std::function<bool(const MapObject&)> ObjectPredicate;
    
    // MARK: mark Node Initialisation
    
    Map(const std::string& path, const std::string& tileSetsRoot);
    
    virtual ~Map();
    
    // MARK: - Coordinate Conversions
    
    ivec2 size() const { return size_; }
    
    Rect frame();
    
    mat33 origin();
    
    vec2 positionForIndexPath(const ivec2& indexPath);
    
    ivec2 indexPathForPosition(const vec2& position);
    
    // MARK: - Layer Access
    
    ARC<MapLayer> layerNamed(const std::string& name);
    
    // Mark: - Objects Accesss
    
    MapObject* objectNamed(const std::string& name);
    
    void enumerateObjects(ObjectBlock block);
    
    void enumerateObjectsMatching(ObjectPredicate predicate, ObjectBlock block);
    
    void enumerateObjectsForType(const std::string& type, ObjectBlock block);
    
    void enumerateObjectsInGroup(const std::string& group, ObjectBlock block);
    
    // MARK: - Rendering stuff
    
    void draw(RendererProtocol& renderer) override;
    
private:
    
    void parseObjects(const json& groupData);
    
    MapOptions                          options_;
    size2                               size_;
    Rect                                frame_;
    std::vector<ARC<MapLayer>>          layers_;
    std::vector<MapObject>              objects_;
    ARC<TextureProtocol>                tileSet_;
    
};

NS_END
