//
//  IterationRange.hpp
//  CompEngine
//
//  Created by Cesar Parent on 05/02/2016.
//  Copyright © 2016 cesarparent. All rights reserved.
//

#ifndef IterationRange_hpp
#define IterationRange_hpp

#include <iostream>

template<typename T>
class __range {
public:
    explicit __range(T endValue) : startValue(0), endValue(endValue), increment(1) { }
    __range(T start, T endValue, T increment = 1) : startValue(start), endValue(endValue), increment(increment) { }
    
    size_t size() const {
        //return endValue,
        return (endValue - startValue)/increment;
    }
    
    class __rangeIterator {
    public:
        friend __range;
        
        __rangeIterator& operator++() {
            value += increment;
            return *this;
        }
        
        __rangeIterator operator++(int) {
            value += increment;
            return __rangeIterator(value - 1, end);
        }
        
        bool operator==(const __rangeIterator& other) const {
            return (value == other.value && end == other.end) ||
            (ended() && other.ended());
        }
        
        bool operator!=(const __rangeIterator& other) const {
            return !(*this == other);
        }
        
        T operator*() const {
            return value;
        }
        
    private:
        __rangeIterator(T end) : value(end), end(end), increment(1) { }
        __rangeIterator(T value, T end, T increment = 1) : value(value), end(end), increment(increment) { }
        bool ended() const {
            return value >= end;
        }
        
        T value;
        T end;
        T increment;
    };
    
    __rangeIterator begin() const {
        return __rangeIterator(startValue, endValue, increment);
    }
    __rangeIterator end() const {
        return __rangeIterator(endValue, increment);
    }
    
private:
    T startValue;
    T endValue;
    T increment;
};

typedef __range<int> _;

#endif /* IterationRange_hpp */
