//
//  JSONMapLayer.hpp
//  RainbowJam16
//
//  Created by Cesar Parent on 21/08/2016.
//  Copyright © 2016 cesarparent. All rights reserved.
//
#pragma once
#include <map>
#include <functional>
#include "../Foundation/CPObject.h"
#include "../Graphics/TextureProtocol.h"
#include "../Graphics/RendererProtocol.h"
#include "MapTile.h"
#include "MapOptions.h"
#include "json.h"
using json = nlohmann::json;

NS_BEGIN

class Map;

class MapLayer : public CPObject {
public:
    
    MapLayer(Map* map, const MapOptions& options, const json& layerData);
    
    MapLayer(const MapLayer& rhs);
    
    virtual ~MapLayer();
    
    MapTile::GID tileGIDAtIndexPath(const ivec2& indexPath);
    
    Rect tileBoundsAtIndexPath(const ivec2& indexPath);
    
    MapTile* tileAtIndexPath(const ivec2& indexPath);
    
    MapTile* firstTileWithGID(MapTile::GID gid);
    
    void enumerateTilesWithGID(MapTile::GID gid, std::function<void(MapTile&, bool&)> block);
    
    void draw(ARC<TextureProtocol> tileSet, RendererProtocol& renderer);
    
    const std::string& name() const { return name_; }
    
private:
    
    Uint64 keyForIndexPath(const ivec2& indexPath);
    
    Map*                map_;
    const MapOptions&   options_;
    std::string         name_;
    bool                hidden_;
    
    std::map<Uint64, MapTile> tiles_;
    
};

NS_END
